/**
 * Copyright 2015 Emmanuel Bourg
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.debian.gradle;

import java.io.File;

import org.gradle.api.internal.file.DefaultTemporaryFileProvider;
import org.gradle.internal.Factory;

/**
 * Create temporary directories into the local .gradle directory using during the Debian build.
 */
class DebianTemporaryFileProvider extends DefaultTemporaryFileProvider {

    public DebianTemporaryFileProvider() {
        super(new Factory<File>() {
            @Override
            public File create() {
                File directory = new File(".gradle/tmp/");
                directory.mkdirs();
                return directory;
            }

        });
    }
}
